--[[
*************************************************************************
    SLTimex: Core.lua
    Author: Taffu of Draka
    Version: 1.1
    Website: http://www.wowinterface.com/portal.php?&uid=24019
    Revision Date: October 12, 2007
    
    This file is part of SLTimex.

    SLTimex is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    SLTimex is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SLTimex.  If not, see <http://www.gnu.org/licenses/>.

*************************************************************************
]]

local L = AceLibrary("AceLocale-2.2"):new("SLT")
local D = AceLibrary("Dewdrop-2.0")
local SM = AceLibrary("SharedMedia-1.0")

local options = { 
    type='group',
    args = {
        header = {
            type = 'header',
            name = L["SLTimex"],
            order = 1,
        },
        space1 = {
            type = 'header',
            name = " ",
            order = 2,
        },
        lock = {
            type = 'toggle',
            name = L["Lock"],
            desc = L["Locks the frame in place."],
            get = "GetLocked",
            set = "SetLocked",
            order = 3,
        },
        space2 = {
            type = 'header',
            name = " ",
            order = 4,
        },
        display = {
            name = L["Display"],
            desc = L["Display Styles."],
            type = 'group',
            order = 5,
            args = {
                bottomclock = {
                    type = 'toggle',
                    name = L["Bottom Clock"],
                    desc = L["Toggles the clock position to the bottom."],
                    get = "IsClockBottom",
                    set = "ToggleClockBottom",
                    order = 1,
                },
                clock24Hour = {
                    type = 'toggle',
                    name = L["24 Hour Clock"],
                    desc = L["Toggles between 24 Hour and 12 Hour style clock."],
                    get = "Is24Hour",
                    set = "Toggle24Hour",
                    order = 2,
                },
                scale = {
                    type = 'range',
                    name = L["Set Scale"],
                    desc = L["Sets the scale."],
                    usage = L["<Range 0.1 to 2.0>"],
                    step = 0.1,
                    min = 0.1,
                    max = 2.0,
                    get = "GetScale",
                    set = "SetScale",
                    order = 3,
                },
                offsetX = {
                    type = 'range',
                    name = L["Offset X"],
                    desc = L["Sets the horizontal frame position."],
                    usage = L["<Range -1000 to 1000>"],
                    step = 1,
                    min = -1000,
                    max = 1000,
                    get = "GetOffsetX",
                    set = "SetOffsetX",
                    order = 4,
                },
                offsetY = {
                    type = 'range',
                    name = L["Offset Y"],
                    desc = L["Sets the vertical frame position."],
                    usage = L["<Range -1000 to 1000>"],
                    step = 1,
                    min = -1000,
                    max = 1000,
                    get = "GetOffsetY",
                    set = "SetOffsetY",
                    order = 5,
                },
                justifytext = {
                    name = L["Justify Text"],
                    desc = L["Sets the text justification for the display."],
                    type = 'group',
                    order = 6,
                    args = {
                        justifytime = {
                            type = 'text',
                            name = L["Justify Time"],
                            desc = L["Sets the text justification for the time display."],
                            usage = L["<LEFT, RIGHT, or CENTER>"],
                            validate = { L["LEFT"], L["RIGHT"], L["CENTER"] },
                            get = "GetTimeJustify",
                            set = "SetTimeJustify",
                            order = 1,
                        },
                        justifystats = {
                            type = 'text',
                            name = L["Justify Stats"],
                            desc = L["Sets the text justification for the stats display."],
                            usage = L["<LEFT, RIGHT, or CENTER>"],
                            validate = { L["LEFT"], L["RIGHT"], L["CENTER"] },
                            get = "GetStatsJustify",
                            set = "SetStatsJustify",
                            order = 2,
                        },
                    },
                },
                setfontsize = {
                    name = L["Font Size"],
                    desc = L["Sets the font sizes."],
                    type = 'group',
                    order = 7,
                    args = {
                        sizetime = {
                            type = 'range',
                            name = L["Time Size"],
                            desc = L["Sets the Time font size."],
                            usage = L["<10-32>"],
                            step = 1,
                            min = 10,
                            max = 32,
                            get = "GetTimeSize",
                            set = "SetTimeSize",
                            order = 1,
                        },
                        sizestats = {
                            type = 'range',
                            name = L["Stats Size"],
                            desc = L["Sets the Stats font size."],
                            usage = L["<10-32>"],
                            step = 1,
                            min = 10,
                            max = 32,
                            get = "GetStatsSize",
                            set = "SetStatsSize",
                            order = 2,
                        },
                    },
                },
                setfont = {
                    type = 'text',
                    name = L["Font"],
                    desc = L["Choose available Font Style."],
                    usage = L["<Font Name>"],
                    validate = SM:List("font"),
                    get = "GetCurFont",
                    set = "SetCurFont",
                    order = 8,
                },
            },
        },
        seperator = {
            name = L["Seperator"],
            desc = L["Seperator Styles."],
            type = 'group',
            order = 6,
            args = {
                stats = {
                    type = 'text',
                    name = L["Stats"],
                    desc = L["Sets the character used to seperate the stats text line."],
                    usage = L["<Seperator Character>"],
                    get = "GetStatSep",
                    set = "SetStatSep",
                    order = 2,
                },
                clock = {
                    type = 'text',
                    name = L["Clock"],
                    desc = L["Sets the character used to seperate Hour and Minute on the clock."],
                    usage = L["<Seperator Character>"],
                    get = "GetClockSep",
                    set = "SetClockSep",
                    order = 1,
                },
            },
        },
        modules = {
            name = L["Modules"],
            desc = L["Stats Modules."],
            type = 'group',
            order = 7,
            args = {
                toggleMemory = {
                    type = 'toggle',
                    name = L["Toggle Memory"],
                    desc = L["Toggles the memory stat on and off."],
                    get = "IsMemShown",
                    set = "ToggleMemShown",
                    order = 1,
                },
                toggleLatency = {
                    type = 'toggle',
                    name = L["Toggle Latency"],
                    desc = L["Toggles the latency stat on and off."],
                    get = "IsLagShown",
                    set = "ToggleLagShown",
                    order = 2,
                },
                toggleFPS = {
                    type = 'toggle',
                    name = L["Toggle FPS"],
                    desc = L["Toggles the FPS stat on and off."],
                    get = "IsFPSShown",
                    set = "ToggleFPSShown",
                    order = 3,
                },
            },
        },
        space3 = {
            type = 'header',
            name = " ",
            order = 8,
        },
        garbagedump = {
            type = 'execute',
            name = L["Garbage Dump"],
            desc = L["Performs a Garbage Collection"],
            func = function() collectgarbage("collect") end,
            order = 9,
        },
    },
}

SLT = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")

function SLT:OnInitialize()
    SLT:RegisterChatCommand(L["Slash-Commands"], options)
    SLT:RegisterDB("SLTVar", "SLTVarChar")
    SLT:RegisterDefaults("profile", {
        textTimeJustH = "RIGHT",
        textStatJustH = "RIGHT",
        frameOffsetX = 250,
        frameOffsetY = 0,
        statSep = " | ",
        clockSep = ":",
        frameScale = 1.0,
        frameLock = false,
        hour24Clock = false,
        showFPS = true,
        showMEM = true,
        showLAG = true,
        clockBottom = false,
        curFont = "Timeless",
        curTimeSize = 26,
        curStatSize = 16,
    })
    
    local localPath = [[Interface\Addons\SLTimex\Fonts\]]
    SM:Register("font", "Carolingia", localPath.."Carolingia.ttf")
    SM:Register("font", "Digital", localPath.."Digital.ttf")
    SM:Register("font", "Hurtmold", localPath.."Hurtmold.ttf")
    SM:Register("font", "JMADH", localPath.."JMADH.ttf")
    SM:Register("font", "Prozak", localPath.."Prozak.ttf")
    SM:Register("font", "Quake", localPath.."Quake.ttf")
    SM:Register("font", "Stonehenge", localPath.."Stonehenge.ttf")
    SM:Register("font", "Timeless", localPath.."Timeless.ttf")

    self:CreateSLTFrame()
end

function SLT:OnEnable()
    SLTFrame:Show()
    self:ScheduleRepeatingEvent(self.UpdateFrame, 1, self)
end

function SLT:OnDisable()
    SLTFrame:Hide()
end

function SLT:CreateSLTFrame()
    local frame = CreateFrame("Frame", "SLTFrame", UIParent)
    frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
    frame:SetWidth(220)
    frame:SetHeight(64)
    frame:SetFrameStrata("BACKGROUND")
    frame:SetMovable(true)
	frame:RegisterForDrag("LeftButton")
    frame:EnableMouse(true)
    frame:SetScript("OnDragStart", 
        function()
            if not self.db.profile.frameLock then 
                this:StartMoving() 
            end
        end)
	frame:SetScript("OnDragStop",
		function()
			this:StopMovingOrSizing()
			local p, rt, rp, x, y = this:GetPoint()
			self.db.profile.frameOffsetX = math.floor(x)
			self.db.profile.frameOffsetY = math.floor(y)
        end)
    frame:SetScript("OnEnter", function(this)
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT")
		GameTooltip:SetText("SLTimex v1.1", 0, 1.0, 0)
        GameTooltip:AddLine("<Right-click> for menu.", 1.0, 1.0, 1.0)
		GameTooltip:Show()
	end)
	frame:SetScript("OnLeave", function()
        GameTooltip:Hide()
    end)
    
    frame.time = frame:CreateFontString("SLTTime", "OVERLAY")
    frame.time:SetJustifyH("Right")
    frame.time:SetShadowColor(0, 0, 0, 0.8)
    frame.time:SetShadowOffset(1, -1)
    
    frame.stats = frame:CreateFontString("SLTStats", "OVERLAY")
    frame.stats:SetJustifyH("Right")
    frame.stats:SetShadowColor(0, 0, 0, 0.8)
    frame.stats:SetShadowOffset(1, -1)
    
    D:Register(frame, 'children', function() D:FeedAceOptionsTable(options) end, 'cursorX', true, 'cursorY', true)
end

function SLT:UpdateFrame()
    SLTFrame:SetPoint("CENTER", UIParent, "CENTER", self.db.profile.frameOffsetX, self.db.profile.frameOffsetY)
    SLTFrame:SetScale(self.db.profile.frameScale)
    
    SLTTime:SetJustifyH(self.db.profile.textTimeJustH)
    SLTStats:SetJustifyH(self.db.profile.textStatJustH)
    
    local localSMFont = SM:Fetch("font", self.db.profile.curFont)
    SLTTime:SetFont(localSMFont, self.db.profile.curTimeSize)
    SLTStats:SetFont(localSMFont, self.db.profile.curStatSize)
    
    if self.db.profile.clockBottom then
        SLTStats:SetPoint("TOP", "SLTFrame", "TOP", 0, 0)
        SLTTime:SetPoint("TOP", "SLTStats", "BOTTOM", 0, 0)
    else
        SLTTime:SetPoint("TOP", "SLTFrame", "TOP", 0, 0)
        SLTStats:SetPoint("TOP", "SLTTime", "BOTTOM", 0, 0)
    end
    
    SLTFrame:SetHeight(SLTTime:GetHeight() + SLTStats:GetHeight())
    
    SLTTime:SetWidth(220)
    SLTStats:SetWidth(220)
    
    local hour = date("%H")
    local minute = date("%M")
    local clockSeperator = self.db.profile.clockSep
    
    if self.db.profile.hour24Clock then
        SLTTime:SetText(hour..clockSeperator..minute)
    else
        local fixHour = math.floor(hour-hour)+date("%H")

        if (fixHour > 11 and fixHour ~= 24) then
            newHour = math.floor(fixHour-12)
            SLTTime:SetText(newHour..clockSeperator..minute.."pm")
        else
            newHour = fixHour
            SLTTime:SetText(newHour..clockSeperator..minute.."am")
        end
    end
    
    local down, up, lag = GetNetStats()
    local fps = GetFramerate()
    local mem = floor(collectgarbage("count") / 1024)
    local statSep = self.db.profile.statSep
    
    if self.db.profile.showMEM and self.db.profile.showLAG and self.db.profile.showFPS then
        SLTStats:SetText(mem.."mb|cffffea00"..statSep.."|r"..lag.."ms|cffffea00"..statSep.."|r"..floor(fps).."fps") -- All three showing
    elseif not self.db.profile.showMEM and self.db.profile.showLAG and self.db.profile.showFPS then
        SLTStats:SetText(lag.."ms|cffffea00"..statSep.."|r"..floor(fps).."fps") -- LAG & FPS showing
    elseif self.db.profile.showMEM and not self.db.profile.showLAG and self.db.profile.showFPS then
        SLTStats:SetText(mem.."mb|cffffea00"..statSep.."|r"..floor(fps).."fps") -- MEM & FPS showing
    elseif self.db.profile.showMEM and self.db.profile.showLAG and not self.db.profile.showFPS then
        SLTStats:SetText(mem.."mb|cffffea00"..statSep.."|r"..lag.."ms") -- MEM & LAG showing
    elseif not self.db.profile.showMEM and not self.db.profile.showLAG and self.db.profile.showFPS then
        SLTStats:SetText(floor(fps).."fps") -- Just FPS showing
    elseif not self.db.profile.showMEM and self.db.profile.showLAG and not self.db.profile.showFPS then
        SLTStats:SetText(lag.."ms") -- Just LAG showing
    elseif self.db.profile.showMEM and not self.db.profile.showLAG and not self.db.profile.showFPS then
        SLTStats:SetText(mem.."mb") -- Just MEM showing
    elseif not self.db.profile.showMEM and not self.db.profile.showLAG and not self.db.profile.showFPS then
        SLTStats:SetText("") -- Nothing showing
    end
end

-- Options stuff
function SLT:IsMemShown()
    return self.db.profile.showMEM
end

function SLT:ToggleMemShown()
    self.db.profile.showMEM = not self.db.profile.showMEM
    self:UpdateFrame()
end

function SLT:IsLagShown()
    return self.db.profile.showLAG
end

function SLT:ToggleLagShown()
    self.db.profile.showLAG = not self.db.profile.showLAG
    self:UpdateFrame()
end

function SLT:IsFPSShown()
    return self.db.profile.showFPS
end

function SLT:ToggleFPSShown()
    self.db.profile.showFPS = not self.db.profile.showFPS
    self:UpdateFrame()
end

function SLT:GetLocked()
    return self.db.profile.frameLock
end

function SLT:SetLocked()
    self.db.profile.frameLock = not self.db.profile.frameLock
    self:UpdateFrame()
end

function SLT:Is24Hour()
    return self.db.profile.hour24Clock
end

function SLT:Toggle24Hour()
    self.db.profile.hour24Clock = not self.db.profile.hour24Clock
    self:UpdateFrame()
end

function SLT:GetScale()
    return self.db.profile.frameScale
end

function SLT:SetScale(newValue)
    self.db.profile.frameScale = newValue
    self:UpdateFrame()
end

function SLT:GetStatSep()
    return self.db.profile.statSep
end

function SLT:SetStatSep(newValue)
    self.db.profile.statSep = newValue
    self:UpdateFrame()
end

function SLT:GetClockSep()
    return self.db.profile.clockSep
end

function SLT:SetClockSep(newValue)
    self.db.profile.clockSep = newValue
    self:UpdateFrame()
end

-- Text Justification Start

function SLT:GetTimeJustify()
    return self.db.profile.textTimeJustH
end

function SLT:SetTimeJustify(newValue)
    self.db.profile.textTimeJustH = newValue
    self:UpdateFrame()
end

function SLT:GetStatsJustify()
    return self.db.profile.textStatJustH
end

function SLT:SetStatsJustify(newValue)
    self.db.profile.textStatJustH = newValue
    self:UpdateFrame()
end

-- Text Justification End

function SLT:GetOffsetX()
    return self.db.profile.frameOffsetX
end

function SLT:SetOffsetX(newValue)
    self.db.profile.frameOffsetX = newValue
    self:UpdateFrame()
end

function SLT:GetOffsetY()
    return self.db.profile.frameOffsetY
end

function SLT:SetOffsetY(newValue)
    self.db.profile.frameOffsetY = newValue
    self:UpdateFrame()
end

function SLT:IsClockBottom()
    return self.db.profile.clockBottom
end

function SLT:ToggleClockBottom()
    self.db.profile.clockBottom = not self.db.profile.clockBottom
    self:UpdateFrame()
end

-- Font Properties Start

function SLT:GetCurFont()
    return self.db.profile.curFont
end

function SLT:SetCurFont(newValue)
    self.db.profile.curFont = newValue
    self:UpdateFrame()
end

function SLT:GetTimeSize()
    return self.db.profile.curTimeSize
end

function SLT:SetTimeSize(newValue)
    self.db.profile.curTimeSize = newValue
    self:UpdateFrame()
end

function SLT:GetStatsSize()
    return self.db.profile.curStatSize
end

function SLT:SetStatsSize(newValue)
    self.db.profile.curStatSize = newValue
    self:UpdateFrame()
end

-- Font Properties End